@@ -28,6 +28,7 @@
 #include "splines2Armadillo/CSpline.h"
 #include "splines2Armadillo/BernsteinPoly.h"
 #include "splines2Armadillo/NaturalSpline.h"
-#include "splines2Armadillo/PeriodicMSpline.h"
+#include "splines2Armadillo/NaturalSplineK.h"
+#include "splines2Armadillo/PeriodicSpline.h"
 
 #endif
